unit PakExplr;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TPakFileType = (ptAll, ptDemos, ptEnv, ptMaps, ptModels, ptPics, ptSound, ptSprites, ptTextures);

  TPakExplr = class(TComponent)
  private
    FPakFile: String;
    FResourceList: TStrings;
    FResourceType: TPakFileType;
  protected
    procedure SetResourceList(Value: TStrings);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean; dynamic;
  published
    property PakFile: String read FPakFile write FPakFile;
    property ResourceList: TStrings read FResourceList write SetResourceList;
    property ResourceType: TPakFileType read FResourceType write FResourceType default ptAll;
  end;

procedure Register;

implementation


procedure Register;
begin
  RegisterComponents('Angstrom', [TPakExplr]);
end;

{ ----- }

function PreTok(s,token:String):String;
begin
  result:= Copy(s,1,Pos(token,s)-1);
end;

function PostTok(s,token:String):String;
begin
  result:= Copy(s,Pos(token,s)+1,Length(s));
end;

function HexToInt(HexStr: String): LongInt;
var
  s : string;
begin
  s := '$' + HexStr;
  result := StrToInt(s);
end;

function IntToHex(DecValue: Integer): String;
begin
  result:= Format('%0x', [DecValue]);
end;

{ TPakExplr }

constructor TPakExplr.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FResourceList:= TStringList.Create;
end;

destructor TPakExplr.Destroy;
begin
  FResourceList.Free;
  inherited Destroy;
end;

procedure TPakExplr.SetResourceList(Value: TStrings);
begin
  FResourceList.Assign(Value);
end;

function TPakExplr.Execute: Boolean;
var
  FromF: file;
  NumRead: Integer;
  Buf: array[1..4] of Char;
  Buf2: array[1..4] of Byte;
  Buf3: array[1..56] of Char;
  hexstr,s: String;
  i: Integer;
  x: byte;
  offset: LongInt;
  ALength: Integer;
  recnum: Integer;
  key: String;
begin
  if FileExists(FPakFile) then begin
    result:= True;
    AssignFile(FromF, FPakFile);
    Reset(FromF, 1);

    {Skip over pack header}
    BlockRead(FromF, Buf, SizeOf(Buf), NumRead);

    {First read offset to directory}
    BlockRead(FromF, Buf2, SizeOf(Buf2), NumRead);
    hexstr:= '';
    for i:= 4 downto 1 do begin
      x:= Buf2[i];
      s:= IntToHex(x);
      HexStr:= HexStr + s;
    end;
    offset:= HexToInt(hexstr);

    {Now get directory length}
    BlockRead(FromF, Buf2, SizeOf(Buf2), NumRead);
    hexstr:= '';
    for i:= 4 downto 1 do begin
      x:= Buf2[i];
      s:= IntToHex(x);
      HexStr:= HexStr + s;
    end;
    Alength:= HexToInt(hexstr);
    Seek(FromF,offset);
    recnum:= ALength div 64;
    FResourceList.Clear;
    for i:= 1 to recnum do begin
      BlockRead(FromF, Buf3, SizeOf(Buf3), NumRead);
      BlockRead(FromF, Buf2, SizeOf(Buf2), NumRead);
      BlockRead(FromF, Buf2, SizeOf(Buf2), NumRead);
      case FResourceType of
        ptDemos:    key:= 'demos';
        ptEnv:      key:= 'env';
        ptMaps:     key:= 'maps';
        ptModels:   key:= 'models';
        ptPics:     key:= 'pics';
        ptSound:    key:= 'sound';
        ptSprites:  key:= 'sprites';
        ptTextures: key:= 'textures';
      end;

      if (FResourceType = ptAll) or (PreTok(Buf3,'/') = key) then
        FResourceList.Add(PostTok(Buf3,'/'));
    end;
    CloseFile(FromF);
  end
  else
    result:= False;
end;


end.
